@@ -17,12 +17,14 @@
 #include <openssl/rand.h>
 #include <openssl/err.h>
 #include <openssl/md5.h>
+#endif /* HAVE_SSL */
 
 ldns_rr *
 ldns_create_empty_rrsig(ldns_rr_list *rrset,
                         ldns_key *current_key)
 {
 	uint32_t orig_ttl;
+	ldns_rr_class orig_class;
 	time_t now;
 	ldns_rr *current_sig;
 	uint8_t label_count;
@@ -34,8 +36,10 @@ ldns_create_empty_rrsig(ldns_rr_list *rrset,
 	
 	/* set the type on the new signature */
 	orig_ttl = ldns_rr_ttl(ldns_rr_list_rr(rrset, 0));
+	orig_class = ldns_rr_get_class(ldns_rr_list_rr(rrset, 0));
 
 	ldns_rr_set_ttl(current_sig, orig_ttl);
+	ldns_rr_set_class(current_sig, orig_class);
 	ldns_rr_set_owner(current_sig, 
 			  ldns_rdf_clone(
 			       ldns_rr_owner(
@@ -50,7 +54,6 @@ ldns_create_empty_rrsig(ldns_rr_list *rrset,
 		   ldns_native2rdf_int32(LDNS_RDF_TYPE_INT32,
 					 orig_ttl));
 	/* the signers name */
-
 	(void)ldns_rr_rrsig_set_signame(
 			current_sig, 
 			ldns_rdf_clone(ldns_key_pubkey_owner(current_key)));
@@ -106,7 +109,7 @@ ldns_create_empty_rrsig(ldns_rr_list *rrset,
 	return current_sig;
 }
 
-
+#ifdef HAVE_SSL
 ldns_rdf *
 ldns_sign_public_buffer(ldns_buffer *sign_buf, ldns_key *current_key)
 {
@@ -129,20 +132,26 @@ ldns_sign_public_buffer(ldns_buffer *sign_buf, ldns_key *current_key)
 		break;
 #ifdef USE_SHA2
 	case LDNS_SIGN_RSASHA256:
-	case LDNS_SIGN_RSASHA256_NSEC3:
 		b64rdf = ldns_sign_public_evp(
 				   sign_buf,
 				   ldns_key_evp_key(current_key),
 				   EVP_sha256());
 		break;
 	case LDNS_SIGN_RSASHA512:
-	case LDNS_SIGN_RSASHA512_NSEC3:
 		b64rdf = ldns_sign_public_evp(
 				   sign_buf,
 				   ldns_key_evp_key(current_key),
 				   EVP_sha512());
 		break;
 #endif /* USE_SHA2 */
+#ifdef USE_GOST
+	case LDNS_SIGN_GOST:
+		b64rdf = ldns_sign_public_evp(
+				   sign_buf,
+				   ldns_key_evp_key(current_key),
+				   EVP_get_digestbyname("md_gost94"));
+		break;
+#endif /* USE_GOST */
 	case LDNS_SIGN_RSAMD5:
 		b64rdf = ldns_sign_public_evp(
 				   sign_buf,
@@ -194,6 +203,8 @@ ldns_sign_public(ldns_rr_list *rrset, ldns_key_list *keys)
 
 	/* make it canonical */
 	for(i = 0; i < ldns_rr_list_rr_count(rrset_clone); i++) {
+		ldns_rr_set_ttl(ldns_rr_list_rr(rrset_clone, i), 
+			ldns_rr_ttl(ldns_rr_list_rr(rrset, 0)));
 		ldns_rr2canonical(ldns_rr_list_rr(rrset_clone, i));
 	}
 	/* sort */
@@ -386,7 +397,6 @@ ldns_sign_public_evp(ldns_buffer *to_sign,
 	return sigdata_rdf;
 }
 
-
 ldns_rdf *
 ldns_sign_public_rsasha1(ldns_buffer *to_sign, RSA *key)
 {
@@ -455,6 +465,23 @@ ldns_sign_public_rsamd5(ldns_buffer *to_sign, RSA *key)
 	ldns_buffer_free(b64sig);
 	return sigdata_rdf;
 }
+#endif /* HAVE_SSL */
+
+static int
+ldns_dnssec_name_has_only_a(ldns_dnssec_name *cur_name)
+{
+	ldns_dnssec_rrsets *cur_rrset;
+	cur_rrset = cur_name->rrsets;
+	while (cur_rrset) {
+		if (cur_rrset->type != LDNS_RR_TYPE_A &&
+			cur_rrset->type != LDNS_RR_TYPE_AAAA) {
+			return 0;
+		} else {
+			cur_rrset = cur_rrset->next;
+		}
+	}
+	return 1;
+}
 
 ldns_status
 ldns_dnssec_zone_mark_glue(ldns_dnssec_zone *zone)
@@ -467,10 +494,7 @@ ldns_dnssec_zone_mark_glue(ldns_dnssec_zone *zone)
 	while (cur_node != LDNS_RBTREE_NULL) {
 		cur_name = (ldns_dnssec_name *) cur_node->data;
 		cur_node = ldns_rbtree_next(cur_node);
-		if (cur_name->rrsets && 
-		    (cur_name->rrsets->type == LDNS_RR_TYPE_A ||
-			cur_name->rrsets->type == LDNS_RR_TYPE_AAAA
-			)) {
+		if (ldns_dnssec_name_has_only_a(cur_name)) {
 			/* assume glue XXX check for zone cur */
 			cur_owner = ldns_rdf_clone(ldns_rr_owner(
 					      cur_name->rrsets->rrs->rr));
@@ -590,6 +614,7 @@ ldns_dnssec_zone_create_nsecs(ldns_dnssec_zone *zone,
 	return LDNS_STATUS_OK;
 }
 
+#ifdef HAVE_SSL
 ldns_status
 ldns_dnssec_zone_create_nsec3s(ldns_dnssec_zone *zone,
 						 ldns_rr_list *new_rrs,
@@ -667,6 +692,7 @@ ldns_dnssec_zone_create_nsec3s(ldns_dnssec_zone *zone,
 	ldns_rr_list_free(nsec3_list);
 	return result;
 }
+#endif /* HAVE_SSL */
 
 ldns_dnssec_rrs *
 ldns_dnssec_remove_signatures(ldns_dnssec_rrs *signatures,
@@ -756,12 +782,43 @@ ldns_dnssec_remove_signatures(ldns_dnssec_rrs *signatures,
 	return base_rrs;
 }
 
+#ifdef HAVE_SSL
 ldns_status
 ldns_dnssec_zone_create_rrsigs(ldns_dnssec_zone *zone,
                                ldns_rr_list *new_rrs,
                                ldns_key_list *key_list,
                                int (*func)(ldns_rr *, void*),
                                void *arg)
+{
+	return ldns_dnssec_zone_create_rrsigs_flg(zone, new_rrs, key_list, 
+		func, arg, 0);
+}
+
+/** If there are KSKs use only them and mark ZSKs unused */
+static void
+ldns_key_list_filter_for_dnskey(ldns_key_list *key_list)
+{
+	int saw_ksk = 0;
+	size_t i;
+	for(i=0; i<ldns_key_list_key_count(key_list); i++)
+		if((ldns_key_flags(ldns_key_list_key(key_list, i))&LDNS_KEY_SEP_KEY)) {
+			saw_ksk = 1;
+			break;
+		}
+	if(!saw_ksk)
+		return;
+	for(i=0; i<ldns_key_list_key_count(key_list); i++)
+		if(!(ldns_key_flags(ldns_key_list_key(key_list, i))&LDNS_KEY_SEP_KEY))
+			ldns_key_set_use(ldns_key_list_key(key_list, i), 0);
+}
+
+ldns_status
+ldns_dnssec_zone_create_rrsigs_flg(ldns_dnssec_zone *zone,
+                               ldns_rr_list *new_rrs,
+                               ldns_key_list *key_list,
+                               int (*func)(ldns_rr *, void*),
+                               void *arg,
+			       int flags)
 {
 	ldns_status result = LDNS_STATUS_OK;
 
@@ -803,6 +860,9 @@ ldns_dnssec_zone_create_rrsigs(ldns_dnssec_zone *zone,
 											key_list,
 											func,
 											arg);
+				if(!(flags&LDNS_SIGN_DNSKEY_WITH_ZSK) &&
+					cur_rrset->type == LDNS_RR_TYPE_DNSKEY)
+					ldns_key_list_filter_for_dnskey(key_list);
 				
 				/* TODO: just set count to zero? */
 				rr_list = ldns_rr_list_new();
@@ -815,9 +875,18 @@ ldns_dnssec_zone_create_rrsigs(ldns_dnssec_zone *zone,
 				
 				/* only sign non-delegation RRsets */
 				/* (glue should have been marked earlier) */
-				if (ldns_rr_list_type(rr_list) != LDNS_RR_TYPE_NS ||
+				if ((ldns_rr_list_type(rr_list) != LDNS_RR_TYPE_NS ||
 					ldns_dname_compare(ldns_rr_list_owner(rr_list),
-					zone->soa->name) == 0) {
+					zone->soa->name) == 0) &&
+					/* OK, there is also the possibility that the record
+					 * is glue, but at the same owner name as other records that
+					 * are not NS nor A/AAAA. Bleh, our current data structure
+					 * doesn't really support that... */
+					!((ldns_rr_list_type(rr_list) == LDNS_RR_TYPE_A ||
+					 ldns_rr_list_type(rr_list) == LDNS_RR_TYPE_AAAA) &&
+					 !ldns_dname_compare(ldns_rr_list_owner(rr_list), zone->soa->name) == 0 &&
+					 ldns_dnssec_zone_find_rrset(zone, ldns_rr_list_owner(rr_list), LDNS_RR_TYPE_NS)
+					 )) {
 
 					siglist = ldns_sign_public(rr_list, key_list);
 					for (i = 0; i < ldns_rr_list_rr_count(siglist); i++) {
@@ -843,6 +912,7 @@ ldns_dnssec_zone_create_rrsigs(ldns_dnssec_zone *zone,
 			}
 			
 			/* sign the nsec */
+			ldns_key_list_set_use(key_list, true);
 			cur_name->nsec_signatures =
 				ldns_dnssec_remove_signatures(cur_name->nsec_signatures,
 										key_list,
@@ -882,6 +952,17 @@ ldns_dnssec_zone_sign(ldns_dnssec_zone *zone,
 				  ldns_key_list *key_list,
 				  int (*func)(ldns_rr *, void *),
 				  void *arg)
+{
+	return ldns_dnssec_zone_sign_flg(zone, new_rrs, key_list, func, arg, 0);
+}
+
+ldns_status
+ldns_dnssec_zone_sign_flg(ldns_dnssec_zone *zone,
+				  ldns_rr_list *new_rrs,
+				  ldns_key_list *key_list,
+				  int (*func)(ldns_rr *, void *),
+				  void *arg,
+				  int flags)
 {
 	ldns_status result = LDNS_STATUS_OK;
 
@@ -900,11 +981,12 @@ ldns_dnssec_zone_sign(ldns_dnssec_zone *zone,
 		}
 	}
 
-	result = ldns_dnssec_zone_create_rrsigs(zone,
-									new_rrs,
-									key_list,
-									func,
-									arg);
+	result = ldns_dnssec_zone_create_rrsigs_flg(zone,
+					new_rrs,
+					key_list,
+					func,
+					arg,
+					flags);
 
 	return result;
 }
@@ -920,6 +1002,23 @@ ldns_dnssec_zone_sign_nsec3(ldns_dnssec_zone *zone,
 					   uint16_t iterations,
 					   uint8_t salt_length,
 					   uint8_t *salt)
+{
+	return ldns_dnssec_zone_sign_nsec3_flg(zone, new_rrs, key_list,
+		func, arg, algorithm, flags, iterations, salt_length, salt, 0);
+}
+
+ldns_status
+ldns_dnssec_zone_sign_nsec3_flg(ldns_dnssec_zone *zone,
+					   ldns_rr_list *new_rrs,
+					   ldns_key_list *key_list,
+					   int (*func)(ldns_rr *, void *),
+					   void *arg,
+					   uint8_t algorithm,
+					   uint8_t flags,
+					   uint16_t iterations,
+					   uint8_t salt_length,
+					   uint8_t *salt,
+					   int signflags)
 {
 	ldns_rr *nsec3, *nsec3params;
 	ldns_status result = LDNS_STATUS_OK;
@@ -936,7 +1035,7 @@ ldns_dnssec_zone_sign_nsec3(ldns_dnssec_zone *zone,
 
 		nsec3 = ((ldns_dnssec_name *)zone->names->root->data)->nsec;
 		if (nsec3 && ldns_rr_get_type(nsec3) == LDNS_RR_TYPE_NSEC3) {
-			// no need to recreate
+			/* no need to recreate */
 		} else {
 			if (!ldns_dnssec_zone_find_rrset(zone,
 									   zone->soa->name,
@@ -970,11 +1069,12 @@ ldns_dnssec_zone_sign_nsec3(ldns_dnssec_zone *zone,
 			}
 		}
 
-		result = ldns_dnssec_zone_create_rrsigs(zone,
-										new_rrs,
-										key_list,
-										func,
-										arg);
+		result = ldns_dnssec_zone_create_rrsigs_flg(zone,
+						new_rrs,
+						key_list,
+						func,
+						arg,
+						signflags);
 	}
 	
 	return result;
@@ -1067,5 +1167,5 @@ ldns_zone_sign_nsec3(ldns_zone *zone, ldns_key_list *key_list, uint8_t algorithm
 
 	return signed_zone;
 }
+#endif /* HAVE_SSL */
 
-#endif
