@@ -150,7 +150,9 @@
      VAR_ADD_HOLDDOWN = 366,
      VAR_DEL_HOLDDOWN = 367,
      VAR_SO_RCVBUF = 368,
-     VAR_EDNS_BUFFER_SIZE = 369
+     VAR_EDNS_BUFFER_SIZE = 369,
+     VAR_PREFETCH = 370,
+     VAR_PREFETCH_KEY = 371
    };
 #endif
 /* Tokens.  */
@@ -266,6 +268,8 @@
 #define VAR_DEL_HOLDDOWN 367
 #define VAR_SO_RCVBUF 368
 #define VAR_EDNS_BUFFER_SIZE 369
+#define VAR_PREFETCH 370
+#define VAR_PREFETCH_KEY 371
 
 
 
@@ -282,7 +286,7 @@ typedef union YYSTYPE
 
 
 /* Line 1676 of yacc.c  */
-#line 286 "util/configparser.h"
+#line 290 "util/configparser.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
