@@ -852,6 +852,10 @@ comm_point_tcp_handle_write(int fd, struct comm_point* c)
 #ifdef EHOSTDOWN
                 else if(error == EHOSTDOWN && verbosity < 2)
                         return 0; /* silence 'host is down' */
+#endif
+#ifdef ETIMEDOUT
+                else if(error == ETIMEDOUT && verbosity < 2)
+                        return 0; /* silence 'connection timed out' */
 #endif
                 else if(error != 0) {
 			log_err("tcp connect: %s", strerror(error));
