@@ -53,10 +53,17 @@ struct config_strlist;
 
 /**
  * This is the TTL to use when a trust anchor fails to prime. A trust anchor
- * will be primed no more often than this interval.
+ * will be primed no more often than this interval.  Used when harden-
+ * dnssec-stripped is off and the trust anchor fails.
  */
 #define NULL_KEY_TTL	900 /* seconds */
 
+/**
+ * TTL for bogus key entries.  When a DS or DNSKEY fails in the chain of
+ * trust the entire zone for that name is blacked out for this TTL.
+ */
+#define BOGUS_KEY_TTL	900 /* seconds */
+
 /** max number of query restarts, number of IPs to probe */
 #define VAL_MAX_RESTART_COUNT 5
 
